@@ -1,4 +1,5 @@
 import warnings
+import yaml
 import numpy as np
 import pandas as pd
 
@@ -6,7 +7,7 @@ from sklearn.cluster import AgglomerativeClustering
 from sklearn.tree import DecisionTreeClassifier
 from sklearn.tree import _tree
 
-from typing import Union, List, Dict
+from typing import List
 from skorecard.bucketers.base_bucketer import BaseBucketer
 from skorecard.bucket_mapping import BucketMapping
 from skorecard.features_bucket_mapping import FeaturesBucketMapping
@@ -72,8 +73,9 @@ class OptimalBucketer(BaseBucketer):
             missing_treatment: Defines how we treat the missing values present in the data.
                 If a string, it must be in ['separate', 'most_risky', 'most_frequent']
                     separate: Missing values get put in a separate 'Other' bucket: `-1`
-                    most_risky: not yet implemented
-                    most_frequent: not yet implemented
+                    most_risky: Missing values are put into the bucket containing the largest percentage of Class 1.
+                    least_risky: Missing values are put into the bucket containing the largest percentage of Class 0.
+                    most_frequent: Missing values are put into the most common bucket.
                 If a dict, it must be of the following format:
                     {"<column name>": <bucket_number>}
                     This bucket number is where we will put the missing values.
@@ -120,9 +122,7 @@ class OptimalBucketer(BaseBucketer):
 
             if feature in self.specials.keys():
                 special = self.specials[feature]
-                X_flt, y_flt = self._filter_specials_for_fit(
-                    X=X[feature], y=y, specials=special
-                )
+                X_flt, y_flt = self._filter_specials_for_fit(X=X[feature], y=y, specials=special)
             else:
                 X_flt, y_flt = X[feature], y
                 special = {}
@@ -172,7 +172,7 @@ class OptimalBucketer(BaseBucketer):
                 splits = binner.splits
 
             # Deal with missing values
-            if self.missing_treatment in ["separate", "most_frequent"]:
+            if self.missing_treatment in ["separate", "most_frequent", "most_risky", "least_risky"]:
                 missing_bucket = None
             if isinstance(self.missing_treatment, dict):
                 missing_bucket = self.missing_treatment.get(feature)
@@ -196,22 +196,8 @@ class OptimalBucketer(BaseBucketer):
                 bucket_mapping=features_bucket_mapping_.get(feature),
             )
 
-            if self.missing_treatment == "most_frequent":
-                most_frequent_row = (
-                    self.bucket_tables_[feature]
-                    .sort_values("Count", ascending=False)
-                    .reset_index(drop=True)
-                    .iloc[0]
-                )
-                if most_frequent_row["label"] != "Missing":
-                    missing_bucket = int(most_frequent_row["bucket_id"])
-                else:
-                    # missings are already the most common bucket, pick the next one
-                    missing_bucket = int(
-                        self.bucket_tables_[feature]
-                        .sort_values("Count", ascending=False)
-                        .reset_index(drop=True)["bucket_id"][1]
-                    )
+            if self.missing_treatment in ["most_frequent", "most_risky", "least_risky"]:
+                missing_bucket = self._find_missing_bucket(feature=feature)
                 # Repeat above procedure now we know the bucket distribution
                 features_bucket_mapping_[feature] = BucketMapping(
                     feature_name=feature,
@@ -286,8 +272,9 @@ class EqualWidthBucketer(BaseBucketer):
             missing_treatment: Defines how we treat the missing values present in the data.
                 If a string, it must be in ['separate', 'most_risky', 'most_frequent']
                     separate: Missing values get put in a separate 'Other' bucket: `-1`
-                    most_risky: not yet implemented
-                    most_frequent: not yet implemented
+                    most_risky: Missing values are put into the bucket containing the largest percentage of Class 1.
+                    least_risky: Missing values are put into the bucket containing the largest percentage of Class 0.
+                    most_frequent: Missing values are put into the most common bucket.
                 If a dict, it must be of the following format:
                     {"<column name>": <bucket_number>}
                     This bucket number is where we will put the missing values.
@@ -320,9 +307,7 @@ class EqualWidthBucketer(BaseBucketer):
 
             if feature in self.specials.keys():
                 special = self.specials[feature]
-                X_flt, y_flt = self._filter_specials_for_fit(
-                    X=X[feature], y=y, specials=special
-                )
+                X_flt, y_flt = self._filter_specials_for_fit(X=X[feature], y=y, specials=special)
             else:
                 X_flt = X[feature]
                 y_flt = y
@@ -341,7 +326,7 @@ class EqualWidthBucketer(BaseBucketer):
                 boundaries = boundaries.tolist()
 
             # Deal with missing values
-            if self.missing_treatment in ["separate", "most_frequent"]:
+            if self.missing_treatment in ["separate", "most_frequent", "most_risky", "least_risky"]:
                 missing_bucket = None
             if isinstance(self.missing_treatment, dict):
                 missing_bucket = self.missing_treatment.get(feature)
@@ -363,22 +348,10 @@ class EqualWidthBucketer(BaseBucketer):
                 bucket_mapping=features_bucket_mapping_.get(feature),
             )
 
-            if self.missing_treatment == "most_frequent":
-                most_frequent_row = (
-                    self.bucket_tables_[feature]
-                    .sort_values("Count", ascending=False)
-                    .reset_index(drop=True)
-                    .iloc[0]
-                )
-                if most_frequent_row["label"] != "Missing":
-                    missing_bucket = int(most_frequent_row["bucket_id"])
-                else:
-                    # missings are already the most common bucket, pick the next one
-                    missing_bucket = int(
-                        self.bucket_tables_[feature]
-                        .sort_values("Count", ascending=False)
-                        .reset_index(drop=True)["bucket_id"][1]
-                    )
+            if self.missing_treatment in ["most_frequent", "most_risky", "least_risky"]:
+                missing_bucket = self._find_missing_bucket(feature=feature)
+
+                assert isinstance(missing_bucket, int)
                 # Repeat above procedure now we know the bucket distribution
                 features_bucket_mapping_[feature] = BucketMapping(
                     feature_name=feature,
@@ -452,8 +425,9 @@ class AgglomerativeClusteringBucketer(BaseBucketer):
             missing_treatment: Defines how we treat the missing values present in the data.
                 If a string, it must be in ['separate', 'most_risky', 'most_frequent']
                     separate: Missing values get put in a separate 'Other' bucket: `-1`
-                    most_risky: not yet implemented
-                    most_frequent: not yet implemented
+                    most_risky: Missing values are put into the bucket containing the largest percentage of Class 1.
+                    least_risky: Missing values are put into the bucket containing the largest percentage of Class 0.
+                    most_frequent: Missing values are put into the most common bucket.
                 If a dict, it must be of the following format:
                     {"<column name>": <bucket_number>}
                     This bucket number is where we will put the missing values.
@@ -490,9 +464,7 @@ class AgglomerativeClusteringBucketer(BaseBucketer):
 
             if feature in self.specials.keys():
                 special = self.specials[feature]
-                X_flt, y_flt = self._filter_specials_for_fit(
-                    X=X[feature], y=y, specials=special
-                )
+                X_flt, y_flt = self._filter_specials_for_fit(X=X[feature], y=y, specials=special)
             else:
                 X_flt = X[feature]
                 y_flt = y
@@ -503,21 +475,13 @@ class AgglomerativeClusteringBucketer(BaseBucketer):
             self.binners[feature] = ab
 
             # Find the boundaries
-            df = pd.DataFrame({"x": X_flt.values, "label": ab.labels_}).sort_values(
-                by="x"
-            )
-            cluster_minimum_values = (
-                df.groupby("label")["x"].min().sort_values().tolist()
-            )
-            cluster_maximum_values = (
-                df.groupby("label")["x"].max().sort_values().tolist()
-            )
+            df = pd.DataFrame({"x": X_flt.values, "label": ab.labels_}).sort_values(by="x")
+            cluster_minimum_values = df.groupby("label")["x"].min().sort_values().tolist()
+            cluster_maximum_values = df.groupby("label")["x"].max().sort_values().tolist()
             # take the mean of the upper boundary of a cluster and the lower boundary of the next cluster
             boundaries = [
                 # Assures numbers are float and not np.float - necessary for serialization
-                float(
-                    np.mean([cluster_minimum_values[i + 1], cluster_maximum_values[i]])
-                )
+                float(np.mean([cluster_minimum_values[i + 1], cluster_maximum_values[i]]))
                 for i in range(len(cluster_minimum_values) - 1)
             ]
 
@@ -525,7 +489,7 @@ class AgglomerativeClusteringBucketer(BaseBucketer):
                 boundaries = boundaries.tolist()
 
             # Deal with missing values
-            if self.missing_treatment in ["separate", "most_frequent"]:
+            if self.missing_treatment in ["separate", "most_frequent", "most_risky", "least_risky"]:
                 missing_bucket = None
             if isinstance(self.missing_treatment, dict):
                 missing_bucket = self.missing_treatment.get(feature)
@@ -547,22 +511,8 @@ class AgglomerativeClusteringBucketer(BaseBucketer):
                 bucket_mapping=features_bucket_mapping_.get(feature),
             )
 
-            if self.missing_treatment == "most_frequent":
-                most_frequent_row = (
-                    self.bucket_tables_[feature]
-                    .sort_values("Count", ascending=False)
-                    .reset_index(drop=True)
-                    .iloc[0]
-                )
-                if most_frequent_row["label"] != "Missing":
-                    missing_bucket = int(most_frequent_row["bucket_id"])
-                else:
-                    # missings are already the most common bucket, pick the next one
-                    missing_bucket = int(
-                        self.bucket_tables_[feature]
-                        .sort_values("Count", ascending=False)
-                        .reset_index(drop=True)["bucket_id"][1]
-                    )
+            if self.missing_treatment in ["most_frequent", "most_risky", "least_risky"]:
+                missing_bucket = self._find_missing_bucket(feature=feature)
                 # Repeat above procedure now we know the bucket distribution
                 features_bucket_mapping_[feature] = BucketMapping(
                     feature_name=feature,
@@ -630,8 +580,9 @@ class EqualFrequencyBucketer(BaseBucketer):
             missing_treatment: Defines how we treat the missing values present in the data.
                 If a string, it must be in ['separate', 'most_risky', 'most_frequent']
                     separate: Missing values get put in a separate 'Other' bucket: `-1`
-                    most_risky: not yet implemented
-                    most_frequent: not yet implemented
+                    most_risky: Missing values are put into the bucket containing the largest percentage of Class 1.
+                    least_risky: Missing values are put into the bucket containing the largest percentage of Class 0.
+                    most_frequent: Missing values are put into the most common bucket.
                 If a dict, it must be of the following format:
                     {"<column name>": <bucket_number>}
                     This bucket number is where we will put the missing values..
@@ -669,28 +620,18 @@ class EqualFrequencyBucketer(BaseBucketer):
 
             if feature in self.specials.keys():
                 special = self.specials[feature]
-                X_flt, y_flt = self._filter_specials_for_fit(
-                    X=X[feature], y=y, specials=special
-                )
+                X_flt, y_flt = self._filter_specials_for_fit(X=X[feature], y=y, specials=special)
             else:
                 X_flt = X[feature]
                 special = {}
             try:
-                _, boundaries = pd.qcut(
-                    X_flt, q=self.n_bins, retbins=True, duplicates="raise"
-                )
+                _, boundaries = pd.qcut(X_flt, q=self.n_bins, retbins=True, duplicates="raise")
             except ValueError:
                 # If there are too many duplicate values (assume a lot of filled missings)
                 # this crashes - the exception drops them.
                 # This means that it will return approximate quantile bins
-                _, boundaries = pd.qcut(
-                    X_flt, q=self.n_bins, retbins=True, duplicates="drop"
-                )
-                warnings.warn(
-                    ApproximationWarning(
-                        "Approximated quantiles - too many unique values"
-                    )
-                )
+                _, boundaries = pd.qcut(X_flt, q=self.n_bins, retbins=True, duplicates="drop")
+                warnings.warn(ApproximationWarning("Approximated quantiles - too many unique values"))
 
             # pd.qcut returns the min & max values of the fits
             # On transform, we use np.digitize, which means new data that is outside of this range
@@ -702,7 +643,7 @@ class EqualFrequencyBucketer(BaseBucketer):
                 boundaries = boundaries.tolist()
 
             # Deal with missing values
-            if self.missing_treatment in ["separate", "most_frequent"]:
+            if self.missing_treatment in ["separate", "most_frequent", "most_risky", "least_risky"]:
                 missing_bucket = None
             if isinstance(self.missing_treatment, dict):
                 missing_bucket = self.missing_treatment.get(feature)
@@ -724,22 +665,9 @@ class EqualFrequencyBucketer(BaseBucketer):
                 bucket_mapping=features_bucket_mapping_.get(feature),
             )
 
-            if self.missing_treatment == "most_frequent":
-                most_frequent_row = (
-                    self.bucket_tables_[feature]
-                    .sort_values("Count", ascending=False)
-                    .reset_index(drop=True)
-                    .iloc[0]
-                )
-                if most_frequent_row["label"] != "Missing":
-                    missing_bucket = int(most_frequent_row["bucket_id"])
-                else:
-                    # missings are already the most common bucket, pick the next one
-                    missing_bucket = int(
-                        self.bucket_tables_[feature]
-                        .sort_values("Count", ascending=False)
-                        .reset_index(drop=True)["bucket_id"][1]
-                    )
+            if self.missing_treatment in ["most_frequent", "most_risky", "least_risky"]:
+                missing_bucket = self._find_missing_bucket(feature=feature)
+
                 # Repeat above procedure now we know the bucket distribution
                 features_bucket_mapping_[feature] = BucketMapping(
                     feature_name=feature,
@@ -831,8 +759,9 @@ class DecisionTreeBucketer(BaseBucketer):
             missing_treatment (str or dict): Defines how we treat the missing values present in the data.
                 If a string, it must be in ['separate', 'most_risky', 'most_frequent']
                     separate: Missing values get put in a separate 'Other' bucket: `-1`
-                    most_risky: not yet implemented
-                    most_frequent: not yet implemented
+                    most_risky: Missing values are put into the bucket containing the largest percentage of Class 1.
+                    least_risky: Missing values are put into the bucket containing the largest percentage of Class 0.
+                    most_frequent: Missing values are put into the most common bucket.
                 If a dict, it must be of the following format:
                     {"<column name>": <bucket_number>}
                     This bucket number is where we will put the missing values.
@@ -879,9 +808,7 @@ class DecisionTreeBucketer(BaseBucketer):
                         f"max_n_bins>= {n_special_bins+2} (currently max_n_bins={self.max_n_bins})"
                     )
 
-                X_flt, y_flt = self._filter_specials_for_fit(
-                    X=X[feature], y=y, specials=special
-                )
+                X_flt, y_flt = self._filter_specials_for_fit(X=X[feature], y=y, specials=special)
             else:
                 X_flt = X[feature]
                 y_flt = y
@@ -909,15 +836,13 @@ class DecisionTreeBucketer(BaseBucketer):
                 binner.fit(X_flt.values.reshape(-1, 1), y_flt)
 
                 # Extract fitted boundaries
-                splits = np.unique(
-                    binner.tree_.threshold[binner.tree_.feature != _tree.TREE_UNDEFINED]
-                )
+                splits = np.unique(binner.tree_.threshold[binner.tree_.feature != _tree.TREE_UNDEFINED])
 
             else:
                 splits = []
 
             # Deal with missing values
-            if self.missing_treatment in ["separate", "most_frequent"]:
+            if self.missing_treatment in ["separate", "most_frequent", "most_risky", "least_risky"]:
                 missing_bucket = None
             if isinstance(self.missing_treatment, dict):
                 missing_bucket = self.missing_treatment.get(feature)
@@ -939,22 +864,8 @@ class DecisionTreeBucketer(BaseBucketer):
                 bucket_mapping=features_bucket_mapping_.get(feature),
             )
 
-            if self.missing_treatment == "most_frequent":
-                most_frequent_row = (
-                    self.bucket_tables_[feature]
-                    .sort_values("Count", ascending=False)
-                    .reset_index(drop=True)
-                    .iloc[0]
-                )
-                if most_frequent_row["label"] != "Missing":
-                    missing_bucket = int(most_frequent_row["bucket_id"])
-                else:
-                    # missings are already the most common bucket, pick the next one
-                    missing_bucket = int(
-                        self.bucket_tables_[feature]
-                        .sort_values("Count", ascending=False)
-                        .reset_index(drop=True)["bucket_id"][1]
-                    )
+            if self.missing_treatment in ["most_frequent", "most_risky", "least_risky"]:
+                missing_bucket = self._find_missing_bucket(feature=feature)
                 # Repeat above procedure now we know the bucket distribution
                 features_bucket_mapping_[feature] = BucketMapping(
                     feature_name=feature,
@@ -1031,7 +942,7 @@ class OrdinalCategoricalBucketer(BaseBucketer):
 
         Args:
             tol (float): the minimum frequency a label should have to be considered frequent.
-                Categories with frequencies lower than tol will be grouped together (in the highest ).
+                Categories with frequencies lower than tol will be grouped together (in the 'other' bucket).
             max_n_categories (int): the maximum number of categories that should be considered frequent.
                 If None, all categories with frequency above the tolerance (tol) will be
                 considered.
@@ -1051,8 +962,9 @@ class OrdinalCategoricalBucketer(BaseBucketer):
             missing_treatment (str or dict): Defines how we treat the missing values present in the data.
                 If a string, it must be in ['separate', 'most_risky', 'most_frequent']
                     separate: Missing values get put in a separate 'Other' bucket: `-1`
-                    most_risky: not yet implemented
-                    most_frequent: not yet implemented
+                    most_risky: Missing values are put into the bucket containing the largest percentage of Class 1.
+                    least_risky: Missing values are put into the bucket containing the largest percentage of Class 0.
+                    most_frequent: Missing values are put into the most common bucket.
                 If a dict, it must be of the following format:
                     {"<column name>": <bucket_number>}
                     This bucket number is where we will put the missing values.
@@ -1090,18 +1002,18 @@ class OrdinalCategoricalBucketer(BaseBucketer):
         features_bucket_mapping_ = {}
         self.bucket_tables_ = {}
 
-        for var in self.variables:
+        for feature in self.variables:
 
             normalized_counts = None
             # Determine the order of unique values
 
-            if var in self.specials.keys():
-                special = self.specials[var]
+            if feature in self.specials.keys():
+                special = self.specials[feature]
                 X_flt, y_flt = self._filter_specials_for_fit(
-                    X=X[var], y=y, specials=special
+                    X=X[feature], y=y, specials=special
                 )
             else:
-                X_flt, y_flt = X[var], y
+                X_flt, y_flt = X[feature], y
                 special = {}
             if not (isinstance(y_flt, pd.Series) or isinstance(y_flt, pd.DataFrame)):
                 y_flt = pd.Series(y_flt)
@@ -1109,17 +1021,16 @@ class OrdinalCategoricalBucketer(BaseBucketer):
             X_flt, y_flt = self._filter_na_for_fit(X=X_flt, y=y_flt)
 
             X_y = pd.concat([X_flt, y_flt], axis=1)
-            X_y.columns = [var, "target"]
+            X_y.columns = [feature, "target"]
 
             if self.encoding_method == "ordered":
                 if y is None:
-                    raise ValueError(
-                        "To use encoding_method=='ordered', y cannot be None."
-                    )
+                    raise ValueError("To use encoding_method=='ordered', y cannot be None.")
+
                 # X_flt["target"] = y_flt
-                normalized_counts = X_y[var].value_counts(normalize=True)
+                normalized_counts = X_y[feature].value_counts(normalize=True)
                 cats = (
-                    X_y.groupby([var])["target"]
+                    X_y.groupby([feature])["target"]
                     .mean()
                     .sort_values(ascending=True)
                     .index
@@ -1127,28 +1038,25 @@ class OrdinalCategoricalBucketer(BaseBucketer):
                 normalized_counts = normalized_counts[cats]
 
             if self.encoding_method == "frequency":
-                normalized_counts = X_y[var].value_counts(normalize=True)
+                normalized_counts = X_y[feature].value_counts(normalize=True)
 
             # Limit number of categories if set.
             normalized_counts = normalized_counts[: self.max_n_categories]
-
             # Remove less frequent categories
             normalized_counts = normalized_counts[normalized_counts >= self.tol]
 
             # Determine Ordinal Encoder based on ordered labels
             # Note we start at 1, to be able to encode missings as 0.
-            mapping = dict(
-                zip(normalized_counts.index, range(0, len(normalized_counts)))
-            )
+            mapping = dict(zip(normalized_counts.index, range(0, len(normalized_counts))))
 
             # Deal with missing values
-            if self.missing_treatment in ["separate", "most_frequent"]:
+            if self.missing_treatment in ["separate", "most_frequent", "most_risky", "least_risky"]:
                 missing_bucket = None
             if isinstance(self.missing_treatment, dict):
-                missing_bucket = self.missing_treatment.get(var)
+                missing_bucket = self.missing_treatment.get(feature)
 
-            features_bucket_mapping_[var] = BucketMapping(
-                feature_name=var,
+            features_bucket_mapping_[feature] = BucketMapping(
+                feature_name=feature,
                 type="categorical",
                 map=mapping,
                 specials=special,
@@ -1156,29 +1064,15 @@ class OrdinalCategoricalBucketer(BaseBucketer):
             )
 
             # Calculate the bucket table
-            self.bucket_tables_[var] = build_bucket_table(
-                X, y, column=var, bucket_mapping=features_bucket_mapping_.get(var)
+            self.bucket_tables_[feature] = build_bucket_table(
+                X, y, column=feature, bucket_mapping=features_bucket_mapping_.get(feature)
             )
 
-            if self.missing_treatment == "most_frequent":
-                most_frequent_row = (
-                    self.bucket_tables_[var]
-                    .sort_values("Count", ascending=False)
-                    .reset_index(drop=True)
-                    .iloc[0]
-                )
-                if most_frequent_row["label"] != "Missing":
-                    missing_bucket = int(most_frequent_row["bucket_id"])
-                else:
-                    # missings are already the most common bucket, pick the next one
-                    missing_bucket = int(
-                        self.bucket_tables_[var]
-                        .sort_values("Count", ascending=False)
-                        .reset_index(drop=True)["bucket_id"][1]
-                    )
+            if self.missing_treatment in ["most_frequent", "most_risky", "least_risky"]:
+                missing_bucket = self._find_missing_bucket(feature=feature)
                 # Repeat above procedure now we know the bucket distribution
-                features_bucket_mapping_[var] = BucketMapping(
-                    feature_name=var,
+                features_bucket_mapping_[feature] = BucketMapping(
+                    feature_name=feature,
                     type="categorical",
                     map=mapping,
                     specials=special,
@@ -1186,8 +1080,8 @@ class OrdinalCategoricalBucketer(BaseBucketer):
                 )
 
                 # Recalculate the bucket table with the new bucket for missings
-                self.bucket_tables_[var] = build_bucket_table(
-                    X, y, column=var, bucket_mapping=features_bucket_mapping_.get(var)
+                self.bucket_tables_[feature] = build_bucket_table(
+                    X, y, column=feature, bucket_mapping=features_bucket_mapping_.get(feature)
                 )
 
         self.features_bucket_mapping_ = FeaturesBucketMapping(features_bucket_mapping_)
@@ -1242,8 +1136,9 @@ class AsIsCategoricalBucketer(BaseBucketer):
             missing_treatment: Defines how we treat the missing values present in the data.
                 If a string, it must be in ['separate', 'most_risky', 'most_frequent']
                     separate: Missing values get put in a separate 'Other' bucket: `-1`
-                    most_risky: not yet implemented
-                    most_frequent: not yet implemented
+                    most_risky: Missing values are put into the bucket containing the largest percentage of Class 1.
+                    least_risky: Missing values are put into the bucket containing the largest percentage of Class 0.
+                    most_frequent: Missing values are put into the most common bucket.
                 If a dict, it must be of the following format:
                     {"<column name>": <bucket_number>}
                     This bucket number is where we will put the missing values.
@@ -1269,15 +1164,14 @@ class AsIsCategoricalBucketer(BaseBucketer):
         features_bucket_mapping_ = {}
         self.bucket_tables_ = {}
 
-        for var in self.variables:
-
-            if var in self.specials.keys():
-                special = self.specials[var]
+        for feature in self.variables:
+            if feature in self.specials.keys():
+                special = self.specials[feature]
                 X_flt, y_flt = self._filter_specials_for_fit(
-                    X=X[var], y=y, specials=special
+                    X=X[feature], y=y, specials=special
                 )
             else:
-                X_flt, y_flt = X[var], y
+                X_flt, y_flt = X[feature], y
                 special = {}
             if not (isinstance(y_flt, pd.Series) or isinstance(y_flt, pd.DataFrame)):
                 y_flt = pd.Series(y_flt)
@@ -1288,13 +1182,13 @@ class AsIsCategoricalBucketer(BaseBucketer):
             mapping = dict(zip(unq, range(0, len(unq))))
 
             # Deal with missing values
-            if self.missing_treatment in ["separate", "most_frequent"]:
+            if self.missing_treatment in ["separate", "most_frequent", "most_risky", "least_risky"]:
                 missing_bucket = None
             if isinstance(self.missing_treatment, dict):
-                missing_bucket = self.missing_treatment.get(var)
+                missing_bucket = self.missing_treatment.get(feature)
 
-            features_bucket_mapping_[var] = BucketMapping(
-                feature_name=var,
+            features_bucket_mapping_[feature] = BucketMapping(
+                feature_name=feature,
                 type="categorical",
                 map=mapping,
                 specials=special,
@@ -1302,29 +1196,16 @@ class AsIsCategoricalBucketer(BaseBucketer):
             )
 
             # Calculate the bucket table
-            self.bucket_tables_[var] = build_bucket_table(
-                X, y, column=var, bucket_mapping=features_bucket_mapping_.get(var)
+            self.bucket_tables_[feature] = build_bucket_table(
+                X, y, column=feature, bucket_mapping=features_bucket_mapping_.get(feature)
             )
 
-            if self.missing_treatment == "most_frequent":
-                most_frequent_row = (
-                    self.bucket_tables_[var]
-                    .sort_values("Count", ascending=False)
-                    .reset_index(drop=True)
-                    .iloc[0]
-                )
-                if most_frequent_row["label"] != "Missing":
-                    missing_bucket = int(most_frequent_row["bucket_id"])
-                else:
-                    # missings are already the most common bucket, pick the next one
-                    missing_bucket = int(
-                        self.bucket_tables_[var]
-                        .sort_values("Count", ascending=False)
-                        .reset_index(drop=True)["bucket_id"][1]
-                    )
+            if self.missing_treatment in ["most_frequent", "most_risky", "least_risky"]:
+                missing_bucket = self._find_missing_bucket(feature=feature)
+
                 # Repeat above procedure now we know the bucket distribution
-                features_bucket_mapping_[var] = BucketMapping(
-                    feature_name=var,
+                features_bucket_mapping_[feature] = BucketMapping(
+                    feature_name=feature,
                     type="categorical",
                     map=mapping,
                     specials=special,
@@ -1332,8 +1213,8 @@ class AsIsCategoricalBucketer(BaseBucketer):
                 )
 
                 # Recalculate the bucket table with the new bucket for missings
-                self.bucket_tables_[var] = build_bucket_table(
-                    X, y, column=var, bucket_mapping=features_bucket_mapping_.get(var)
+                self.bucket_tables_[feature] = build_bucket_table(
+                    X, y, column=feature, bucket_mapping=features_bucket_mapping_.get(feature)
                 )
 
         self.features_bucket_mapping_ = FeaturesBucketMapping(features_bucket_mapping_)
@@ -1390,8 +1271,9 @@ class AsIsNumericalBucketer(BaseBucketer):
             missing_treatment (str or dict): Defines how we treat the missing values present in the data.
                 If a string, it must be in ['separate', 'most_risky', 'most_frequent']
                     separate: Missing values get put in a separate 'Other' bucket: `-1`
-                    most_risky: not yet implemented
-                    most_frequent: not yet implemented
+                    most_risky: Missing values are put into the bucket containing the largest percentage of Class 1.
+                    least_risky: Missing values are put into the bucket containing the largest percentage of Class 0.
+                    most_frequent: Missing values are put into the most common bucket.
                 If a dict, it must be of the following format:
                     {"<column name>": <bucket_number>}
                     This bucket number is where we will put the missing values..
@@ -1424,9 +1306,7 @@ class AsIsNumericalBucketer(BaseBucketer):
 
             if feature in self.specials.keys():
                 special = self.specials[feature]
-                X_flt, y_flt = self._filter_specials_for_fit(
-                    X=X[feature], y=y, specials=special
-                )
+                X_flt, y_flt = self._filter_specials_for_fit(X=X[feature], y=y, specials=special)
             else:
                 X_flt = X[feature]
                 special = {}
@@ -1441,7 +1321,7 @@ class AsIsNumericalBucketer(BaseBucketer):
                 raise NotPreBucketedError(msg)
 
             # Deal with missing values
-            if self.missing_treatment in ["separate", "most_frequent"]:
+            if self.missing_treatment in ["separate", "most_frequent", "most_risky", "least_risky"]:
                 missing_bucket = None
             if isinstance(self.missing_treatment, dict):
                 missing_bucket = self.missing_treatment.get(feature)
@@ -1463,22 +1343,8 @@ class AsIsNumericalBucketer(BaseBucketer):
                 bucket_mapping=features_bucket_mapping_.get(feature),
             )
 
-            if self.missing_treatment == "most_frequent":
-                most_frequent_row = (
-                    self.bucket_tables_[feature]
-                    .sort_values("Count", ascending=False)
-                    .reset_index(drop=True)
-                    .iloc[0]
-                )
-                if most_frequent_row["label"] != "Missing":
-                    missing_bucket = int(most_frequent_row["bucket_id"])
-                else:
-                    # missings are already the most common bucket, pick the next one
-                    missing_bucket = int(
-                        self.bucket_tables_[feature]
-                        .sort_values("Count", ascending=False)
-                        .reset_index(drop=True)["bucket_id"][1]
-                    )
+            if self.missing_treatment in ["most_frequent", "most_risky", "least_risky"]:
+                missing_bucket = self._find_missing_bucket(feature=feature)
                 # Repeat above procedure now we know the bucket distribution
                 features_bucket_mapping_[feature] = BucketMapping(
                     feature_name=feature,
@@ -1551,35 +1417,42 @@ class UserInputBucketer(BaseBucketer):
 
     def __init__(
         self,
-        features_bucket_mapping: Union[Dict, FeaturesBucketMapping],
+        features_bucket_mapping,
         variables: List = [],
     ) -> None:
-        """Initialise the user-defined boundaries with a dictionary.
+        """
+        Initialise the user-defined boundaries with a dictionary.
 
         Notes:
         - features_bucket_mapping is stored without the trailing underscore (_) because it is not fitted.
 
         Args:
-            features_bucket_mapping (dict): Contains the feature name and boundaries defined for this feature.
-                Either dict or FeaturesBucketMapping
+            features_bucket_mapping (Dict, FeaturesBucketMapping, str or Path): Contains the feature name and boundaries
+                defined for this feature.
+                If a dict, it will be converted to an internal FeaturesBucketMapping object.
+                If a string or path, which will attempt to load the file as a yaml and convert to FeaturesBucketMapping object.
             variables (list): The features to bucket. Uses all features in features_bucket_mapping if not defined.
-        """
+        """  # noqa
         # Assigning the variable in the init to the attribute with the same name is a requirement of
         # sklearn.base.BaseEstimator. See the notes in
         # https://scikit-learn.org/stable/modules/generated/sklearn.base.BaseEstimator.html#sklearn.base.BaseEstimator
         self.features_bucket_mapping = features_bucket_mapping
 
-        # Check user defined input for bucketing. If a dict is specified, will auto convert
-        if not isinstance(self.features_bucket_mapping, FeaturesBucketMapping):
-            if not isinstance(self.features_bucket_mapping, dict):
-                raise TypeError(
-                    "'features_bucket_mapping' must be a dict or FeaturesBucketMapping instance"
-                )
-            self.features_bucket_mapping_ = FeaturesBucketMapping(
-                self.features_bucket_mapping
-            )
+        if isinstance(features_bucket_mapping, str):
+            buckets_yaml = yaml.safe_load(open(features_bucket_mapping, "r"))
+            self.features_bucket_mapping_ = FeaturesBucketMapping(buckets_yaml)
+        elif isinstance(features_bucket_mapping, dict):
+            self.features_bucket_mapping_ = FeaturesBucketMapping(features_bucket_mapping)
+        elif isinstance(features_bucket_mapping, FeaturesBucketMapping):
+            self.features_bucket_mapping_ = features_bucket_mapping
         else:
-            self.features_bucket_mapping_ = self.features_bucket_mapping
+            try:
+                buckets_yaml = yaml.safe_load(features_bucket_mapping)
+                self.features_bucket_mapping_ = FeaturesBucketMapping(buckets_yaml)
+            except Exception:
+                raise TypeError(
+                    "'features_bucket_mapping' must be a dict, str, path, or FeaturesBucketMapping instance"
+                )
 
         # If user did not specify any variables,
         # use all the variables defined in the features_bucket_mapping
